#!/usr/bin/perl -w
#########################################################################
# vhtml.pl
# A simple tool for formatting to HTML in a pipe.
#
# Version 0.1 (beta), 2004-08-15
# Copyright (c) 2004 Michael Winkelmann, michael@weltretter.de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

=pod

=head1 NAME

vword.pl - The Voynich word extractor

=head1 SYNOPSIS

C<perl vword.pl [OPTIONS]>

=head1 DESCRIPTION

This little script splits a voynich transcription in single
words and optionally counts the words.

=head1 OPTIONS

=over 4

=item -c

Create a word count statistic instead of just printing the words.

=item -d

Do not print or count words with unreadable characters.

=back

=head1 AUTHOR

Michael Winkelmann, michael@weltretter.de

=cut

use strict;
use Getopt::Std;

my $line;
my $word;
my @words;
my %whash;

our ($opt_c, $opt_d);

getopts('cd');

while($line = <>)
{
    chomp $line;
    next if $line =~ m/^\#/o;
    next unless $line;
    $line =~ s/\<[^\>]*\>//go;
    $line =~ s/\{[^\}]*\}//go;
    $line =~ s/[-=]//go;
    @words= split /[ \,\.\!\%\:]/, $line;
    foreach $word (@words)
    {
	chomp $word;
	next unless $word;
	if($opt_d)
	{
	    next if $word =~ m/\*/o;
	}
	if($opt_c)
	{
	    $whash{$word} += 1;
	}
	else
	{
	    print "$word\n";
	}
    }
}

if($opt_c)
{
    my ($key);
    
    foreach $key (sort { $whash{$b} <=> $whash{$a} } keys %whash)
    {
	printf "%-20s %6d\n", $key, $whash{$key};
    }
}
