#!/usr/bin/perl
=pod

=head1 NAME

wordseq.pl - Word sequence analysis

=head1 SYNOPSIS

C<perl wordseq.pl [filespec]>

=head1 DESCRIPTION

The input for the program must be given as words, every word in
a single line. The program creates a simple word sequence statistic,
each word is followed by the following words and their frequencies.
It is written for the study of the Voynich manuscript.

=head1 OPTIONS

=over 4

=item C<--nosingles> or C<-s>

Do not include single occurencies of a word in the list.

=back

=head1 OUTPUT FORMAT

For each word in the input a line of output will be generated.

The word is the first token, it is followed by a colon.

After the colon there is an entry for each following word, sorted
by decreasing frequency of occurence. The entry contains

=over 4 

=item - The word

=item - The frequency of the word occurences in curly brackets

=item - A dash as end marker for the word

=back

The line is closed by an equal sign.

After the equal sign follows the number of displayed words and, 
if you use the C<-s> option, the number of suppressed words in
curly brackets. 

The last character of the line is the newline character.

The output format is generated in a way which allows you to pipe it
to the C<showeva.pl> tool, so you can read it in the EVA font. For
this usage the frequencies are formatted as EVA inline comments.

=head1 EXAMPLE

C<perl viat.pl -t H | perl vword.pl | perl wordseq.pl | perl showeva.pl ->

Create a word sequence statistic for the VMS transcription of
T. Takahasi and show the statistic in the C<showeva.pl> tool.

=head1 BUGS

The short source is far away from being an example of nice software
engeneering. Do not read it, use it...

=head1 AUTHOR

wordseq.pl was written by Michael Winkelmann, michael@weltretter.de

=cut
    ;
use strict;
use warnings;
use Getopt::Long qw(:config bundling no_ignore_case);

my $word;
my $last;
my %stat;
my ($opt_s);

# Parse options
GetOptions("nosingles|s" => \$opt_s)
    or die "Illegal option used";

# Read input file - each word on its own line
while($word = <>)
{
    chomp $word;
    next if $word =~ m/^\s*$/o;
    $stat{$last}->{$word}++ if($last);
    $last = $word;
}
    
# Create the output - this is the ugliest source I ever wrote...
foreach $word (sort(keys(%stat)))
{
    my ($href, $tmp, $freq, @keyz, $singlesum, $displayed);
    $href = $stat{$word};
    $singlesum = $displayed = 0;
    if($opt_s) {
	foreach $tmp (keys(%{$href})) 
	{
	    delete $href->{$tmp} if($href->{$tmp} <= 1);
	    ++$singlesum;
	}
    }
    print "$word: ";
    foreach $tmp (sort { $href->{$b} <=> $href->{$a} } keys(%{$href}))
    {
	$freq = $href->{$tmp};
	print "$tmp {$freq}- ";
	$displayed += $freq;
    }
    print "=";
    $tmp = $displayed + $singlesum;
    print "{total: $tmp word";
    print "s" unless $tmp == 1;
    if($opt_s)
    {
	if($displayed || $singlesum)
	{
	    print ", $displayed visible" if $displayed;
	    print ", $singlesum hidden" if $singlesum;
	}
    }
    print "}\n";
}
